@@ -21,11 +21,16 @@ class SettingsDialog(QDialog):
         self.lemmatization = QCheckBox("Use lemmatization (Requires restart to take effect)")
         self.lemmatization.setToolTip("Lemmatization means to get the original form of words."
             + "\nFor example, 'books' will be converted to 'book' during lookup if this option is set.")
+        self.lemfreq = QCheckBox("Lemmatize before looking up frequency")
+        self.lemfreq.setToolTip("Lemmatize words before trying to find them in the frequency list."
+            + "\nUse this for frequency lists displayed on FLT.org, but do not use it "
+            + "\nfor frequency lists from Migaku. ")
         self.target_language = QComboBox()
         self.deck_name = QComboBox()
         self.tags = QLineEdit()
         self.dict_source = QComboBox()
         self.dict_source2 = QComboBox()
+        self.freq_source = QComboBox()
         self.gtrans_lang = QComboBox()
         self.note_type = QComboBox()
         self.sentence_field = QComboBox()
@@ -35,6 +40,11 @@ class SettingsDialog(QDialog):
         self.pronunciation_field = QComboBox()
         self.forvo = QCheckBox("Play Forvo pronunciation upon word selection")
 
+        self.web_preset = QComboBox()
+        self.custom_url = QLineEdit()
+        self.custom_url.setText("https://example.com/@@@@")
+        self.custom_url.setEnabled(False)
+
         self.orientation = QComboBox()
         self.text_scale = QSlider(Qt.Horizontal)
 
@@ -51,10 +61,7 @@ class SettingsDialog(QDialog):
         self.text_scale_box_layout.addWidget(self.text_scale)
         self.text_scale_box_layout.addWidget(self.text_scale_label)
 
-        self.orientation.addItems([
-            "Vertical", 
-            "Horizontal", 
-            ])
+        self.orientation.addItems(["Vertical", "Horizontal"])
 
         self.anki_api = QLineEdit()
         self.about_sa = QScrollArea()
@@ -123,14 +130,25 @@ If you find this tool useful, you can donate to these projects.
 
     def setupWidgets(self):
         self.target_language.addItems(code.keys())
+        self.web_preset.addItems([
+            "English Wiktionary",
+            "Monolingual Wiktionary",
+            "Reverso Context",
+            "Tatoeba",
+            "Custom (Enter below)"
+            ])
         self.gtrans_lang.addItems(code.keys())
         
         self.tab1.layout.addRow(self.lemmatization)
+        self.tab1.layout.addRow(self.lemfreq)
         self.tab1.layout.addRow(self.forvo)
         self.tab1.layout.addRow(QLabel("Target language"), self.target_language)
         self.tab1.layout.addRow(QLabel("Dictionary source 1"), self.dict_source)
         self.tab1.layout.addRow(QLabel("Dictionary source 2"), self.dict_source2)
+        self.tab1.layout.addRow(QLabel("Frequency list"), self.freq_source)
         self.tab1.layout.addRow(QLabel("Google translate: To"), self.gtrans_lang)
+        self.tab1.layout.addRow(QLabel("Web lookup preset"), self.web_preset)
+        self.tab1.layout.addRow(QLabel("Custom URL pattern"), self.custom_url)
         self.tab1.layout.addRow(self.importdict)
 
 
@@ -163,13 +181,21 @@ If you find this tool useful, you can donate to these projects.
     def setupAutosave(self):
         self.allow_editing.clicked.connect(self.syncSettings)
         self.lemmatization.clicked.connect(self.syncSettings)
+        self.lemfreq.clicked.connect(self.syncSettings)
         self.forvo.clicked.connect(self.syncSettings)
+        self.freq_source.currentTextChanged.connect(self.syncSettings)
         self.dict_source.currentTextChanged.connect(self.syncSettings)
         self.dict_source.currentTextChanged.connect(self.loadDict2Options)
         self.dict_source2.currentTextChanged.connect(self.syncSettings)
         self.dict_source2.currentTextChanged.connect(self.warnRestart)
         self.target_language.currentTextChanged.connect(self.loadDictionaries)
+        self.target_language.currentTextChanged.connect(self.loadFreqSources)
+        self.target_language.currentTextChanged.connect(self.loadUrl)
         self.target_language.currentTextChanged.connect(self.syncSettings)
+        self.web_preset.currentTextChanged.connect(self.loadUrl)
+        self.web_preset.currentTextChanged.connect(self.syncSettings)
+        self.custom_url.textChanged.connect(self.syncSettings)
+        self.gtrans_lang.currentTextChanged.connect(self.loadUrl)
         self.gtrans_lang.currentTextChanged.connect(self.syncSettings)
         self.deck_name.currentTextChanged.connect(self.syncSettings)
         self.tags.editingFinished.connect(self.syncSettings)
@@ -193,15 +219,17 @@ If you find this tool useful, you can donate to these projects.
         self.orientation.currentTextChanged.connect(self.syncSettings)
 
     def loadDictionaries(self):
+        custom_dicts = self.settings.value("custom_dicts", [], type=list)
         self.dict_source.clear()
-        dicts = getDictsForLang(code[self.target_language.currentText()])
+        dicts = getDictsForLang(code[self.target_language.currentText()], custom_dicts)
         self.dict_source.addItems(dicts)
 
     def loadDict2Options(self):
+        custom_dicts = self.settings.value("custom_dicts", [], type=list)
         curtext = self.dict_source2.currentText()
         self.dict_source2.blockSignals(True)
         self.dict_source2.clear()
-        dicts = getDictsForLang(code[self.target_language.currentText()])
+        dicts = getDictsForLang(code[self.target_language.currentText()], custom_dicts)
         self.dict_source2.addItem("Disabled")
         for item in dicts:
             if self.dict_source.currentText() != item:
@@ -211,11 +239,38 @@ If you find this tool useful, you can donate to these projects.
             self.dict_source2.setCurrentText(curtext)
         self.dict_source2.blockSignals(False)
 
+    def loadFreqSources(self):
+        custom_dicts = self.settings.value("custom_dicts", [], type=list)
+        sources = getFreqlistsForLang(code[self.target_language.currentText()], custom_dicts)
+        self.freq_source.clear()
+        self.freq_source.addItem("Disabled")
+        for item in sources:
+            self.freq_source.addItem(item)
+
+    def loadUrl(self):
+        langfull = self.settings.value("target_language", "English")
+        tolangfull = self.settings.value("gtrans_lang", "English")
+        lang = code[langfull]
+        tolang = code[tolangfull]
+        self.presets = bidict({
+            "English Wiktionary": "https://en.wiktionary.org/wiki/@@@@#" + langfull, 
+            "Monolingual Wiktionary": f"https://{lang}.wiktionary.org/wiki/@@@@",
+            "Reverso Context": f"https://context.reverso.net/translation/{langfull.lower()}-{tolangfull.lower()}/@@@@",
+            "Tatoeba": "https://tatoeba.org/en/sentences/search?query=@@@@"
+            })
+
+        if self.web_preset.currentText() == "Custom (Enter below)":
+            self.custom_url.setEnabled(True)
+            self.custom_url.setText(self.settings.value("custom_url"))
+        else:
+            self.custom_url.setEnabled(False)
+            self.custom_url.setText(self.presets[self.web_preset.currentText()])
 
     def loadSettings(self):
         self.forvo.setChecked(self.settings.value("forvo", False, type=bool))
         self.allow_editing.setChecked(self.settings.value("allow_editing", True, type=bool))
         self.lemmatization.setChecked(self.settings.value("lemmatization", True, type=bool))
+        self.lemfreq.setChecked(self.settings.value("lemfreq", False, type=bool))
         self.orientation.setCurrentText(self.settings.value("orientation", "Vertical"))
         self.text_scale.setValue(self.settings.value("text_scale", 100, type=int))
         self.target_language.setCurrentText(self.settings.value("target_language"))
@@ -223,16 +278,21 @@ If you find this tool useful, you can donate to these projects.
         self.dict_source.setCurrentText(self.settings.value("dict_source", "Wiktionary (English)"))
         self.loadDict2Options()
         self.dict_source2.setCurrentText(self.settings.value("dict_source2", "Wiktionary (English)"))
+        self.loadFreqSources()
+        self.web_preset.setCurrentText(self.settings.value("web_preset", "English Wiktionary"))
+        self.loadUrl()
+        self.freq_source.setCurrentText(self.settings.value("freq_source", "Disabled"))
         self.gtrans_lang.setCurrentText(self.settings.value("gtrans_lang", "English"))
         self.anki_api.setText(self.settings.value("anki_api", "http://localhost:8765"))
         self.tags.setText(self.settings.value("tags", "ssmtool"))
         api = self.anki_api.text()
+        self.web_preset.setCurrentText(self.settings.value("web_preset"))
 
         self.host.setText(self.settings.value("host", "127.0.0.1"))
         self.port.setValue(self.settings.value("port", 39284, type=int))
 
         try:
-            print("API version is: ", getVersion(api))
+            _ = getVersion(api)
         except Exception as e:
             self.errorNoConnection(e)
             return
@@ -249,10 +309,9 @@ If you find this tool useful, you can donate to these projects.
         self.loadFields()
 
     def loadFields(self):
-        print("loading fields")
         api = self.anki_api.text()
         try:
-            print("API version is: ", getVersion(api))
+            _ = getVersion(api)
         except Exception as e:
             self.errorNoConnection(e)
             return
@@ -322,15 +381,19 @@ If you find this tool useful, you can donate to these projects.
         self.settings.setValue("forvo", self.forvo.isChecked())
         self.settings.setValue("allow_editing", self.allow_editing.isChecked())
         self.settings.setValue("lemmatization", self.lemmatization.isChecked())
+        self.settings.setValue("lemfreq", self.lemfreq.isChecked())
         self.settings.setValue("orientation", self.orientation.currentText())
         self.settings.setValue("target_language", self.target_language.currentText())
         self.settings.setValue("dict_source", self.dict_source.currentText())
         self.settings.setValue("dict_source2", self.dict_source2.currentText())
+        self.settings.setValue("freq_source", self.freq_source.currentText())
         self.settings.setValue("gtrans_lang", self.gtrans_lang.currentText())
         self.settings.setValue("anki_api", self.anki_api.text())
         self.settings.setValue("host", self.host.text())
         self.settings.setValue("port", self.port.value())
         self.settings.setValue("text_scale", self.text_scale.value())
+        self.settings.setValue("web_preset", self.web_preset.currentText())
+        self.settings.setValue("custom_url", self.custom_url.text())
         try:
             api = self.anki_api.text()
             getVersion(api)
