@@ -2,6 +2,7 @@ import sys
 from PyQt5.QtWidgets import *
 from PyQt5.QtCore import *
 from PyQt5.QtGui import *
+from PyQt5.Qt import QDesktopServices, QUrl
 from os import path
 import functools
 import platform
@@ -67,6 +68,7 @@ class DictionaryWindow(QMainWindow):
             self.setupWidgetsV()
         else:
             self.setupWidgetsH()
+        self.setupButtons()
         self.startServer()
         self.initTimer()
         self.updateAnkiButtonState()
@@ -116,6 +118,11 @@ class DictionaryWindow(QMainWindow):
         self.setStatusBar(self.bar)
         self.stats_label = QLabel()
 
+        self.web_button = QPushButton("Open webpage")
+        self.freq_display = QLCDNumber()
+        self.freq_display.setSegmentStyle(QLCDNumber.Flat)
+        self.freq_display.display(0)
+
         self.sentence.setReadOnly(not (self.settings.value("allow_editing", True, type=bool)))
         self.definition.setReadOnly(not (self.settings.value("allow_editing", True, type=bool)))
 
@@ -138,7 +145,9 @@ class DictionaryWindow(QMainWindow):
         else:
             self.layout.addWidget(self.lookup_button, 4, 1, 1, 2)
         
-        self.layout.addWidget(QLabel("Definition"), 6, 0, 1, 3)
+        self.layout.addWidget(QLabel("Definition"), 6, 0)
+        self.layout.addWidget(self.freq_display, 6, 1)
+        self.layout.addWidget(self.web_button, 6, 2)
         self.layout.addWidget(self.word, 5, 0, 1, 3)
         if self.settings.value("dict_source2", "Disabled") != "Disabled":
             self.layout.addWidget(self.definition, 7, 0, 2, 3)
@@ -156,9 +165,12 @@ class DictionaryWindow(QMainWindow):
         self.layout.addWidget(self.toanki_button, 13, 0, 1, 3)
         self.layout.addWidget(self.config_button, 14, 0, 1, 3)
 
+    def setupButtons(self):
         self.lookup_button.clicked.connect(lambda _: self.lookupClicked(True))
         self.lookup_exact_button.clicked.connect(lambda _: self.lookupClicked(False))
 
+        self.web_button.clicked.connect(self.onWebButton)
+
         self.config_button.clicked.connect(self.configure)
         self.toanki_button.clicked.connect(self.createNote)
         self.read_button.clicked.connect(lambda _: self.clipboardChanged(True))
@@ -183,6 +195,7 @@ class DictionaryWindow(QMainWindow):
         self.layout.setColumnStretch(4, 5)
 
         self.layout.addWidget(self.label_sentence, 1, 0)
+        self.layout.addWidget(self.freq_display, 1, 1)
         self.layout.addWidget(self.undo_button, 6, 0)
         self.layout.addWidget(self.read_button, 6, 1)
 
@@ -196,6 +209,7 @@ class DictionaryWindow(QMainWindow):
             self.layout.addWidget(self.lookup_button, 1, 4, 2, 1)
         
         self.layout.addWidget(QLabel("Definition"), 1, 3)
+        self.layout.addWidget(self.web_button, 1, 4)
         self.layout.addWidget(self.word, 2, 2, 1, 1)
         if self.settings.value("dict_source2", "Disabled") != "Disabled":
             print(self.settings.value("dict_source2", "Disabled") != "Disabled")
@@ -211,18 +225,6 @@ class DictionaryWindow(QMainWindow):
 
         self.layout.addWidget(self.toanki_button, 6, 3, 1, 1)
         self.layout.addWidget(self.config_button, 6, 4, 1, 1)
-
-        self.lookup_button.clicked.connect(lambda _: self.lookupClicked(True))
-        self.lookup_exact_button.clicked.connect(lambda _: self.lookupClicked(False))
-
-        self.config_button.clicked.connect(self.configure)
-        self.toanki_button.clicked.connect(self.createNote)
-        self.read_button.clicked.connect(lambda _: self.clipboardChanged(True))
-
-        self.sentence.textChanged.connect(self.updateAnkiButtonState)
-        self.undo_button.clicked.connect(self.undo)
-
-        self.bar.addPermanentWidget(self.stats_label)
    
     def updateAnkiButtonState(self, forceDisable=False):
         if self.sentence.toPlainText() == "" or forceDisable:
@@ -253,6 +255,10 @@ class DictionaryWindow(QMainWindow):
 
         return target
 
+    def onWebButton(self):
+        url = self.settings.value("custom_url").replace("@@@@", self.word.text())
+        QDesktopServices.openUrl(QUrl(url))
+
     def lookupClicked(self, use_lemmatize=True):
         if self.forvo_scraping:
             return
@@ -260,8 +266,8 @@ class DictionaryWindow(QMainWindow):
         self.updateAnkiButtonState()
         if target == "":
             return
-        result = self.lookup(target, use_lemmatize)
-        self.setState(result)
+        self.lookupSet(target, use_lemmatize)
+
 
     def setState(self, state):
         self.word.setText(state['word'])
@@ -309,16 +315,24 @@ class DictionaryWindow(QMainWindow):
             sentence = preprocess_clipboard(copyobj['sentence'], lang)
             self.setSentence(sentence)
             self.setWord(target)
-            result = self.lookup(target)
-            self.setState(result)
+            self.lookupSet(target)
         elif is_oneword(preprocess_clipboard(text, lang)):
             self.setSentence(word:=preprocess_clipboard(text, lang))
             self.setWord(word)
-            result = self.lookup(text)
-            self.setState(result)
+            self.lookupSet(text)
         else:
             self.setSentence(preprocess_clipboard(text, lang))
 
+    def lookupSet(self, word, use_lemmatize=True):
+        result = self.lookup(word, use_lemmatize)
+        self.setState(result)
+        QCoreApplication.processEvents()
+        self.audio_path = None
+        if self.settings.value("forvo", False, type=bool) and not self.forvo_scraping:
+            self.forvo_scraping = True
+            self.audio_path = play_forvo(word, code[self.settings.value("target_language")])
+            self.forvo_scraping = False
+
     def lookup(self, word, use_lemmatize=True):
         """
         Look up a word and return a dict with the lemmatized form (if enabled)
@@ -327,17 +341,20 @@ class DictionaryWindow(QMainWindow):
         TL = self.settings.value("target_language", "English") 
         self.prev_states.append(self.getState())
         lemmatize = use_lemmatize and self.settings.value("lemmatization", True, type=bool)
+        lemfreq = self.settings.value("lemfreq", True, type=bool)
         short_sign = "Y" if lemmatize else "N"
         language = code[TL] #This is in two letter code
         gtrans_lang = self.settings.value("gtrans_lang", "English")
         dictname = self.settings.value("dict_source", "Wiktionary (English)")
+        freqname = self.settings.value("freq_source", "Disabled")
         word = re.sub('[«»…()\[\]]*', "", word)
-        self.audio_path = None
+        if freqname != "Disabled":
+            try:
+                freq = getFreq(word, language, lemfreq, freqname)
+            except TypeError:
+                freq = -1
+            self.freq_display.display(freq)
         self.status(f"L: '{word}' in '{language}', lemma: {short_sign}, from {dictionaries.get(dictname, dictname)}")
-        if self.settings.value("forvo", False, type=bool) and not self.forvo_scraping:
-            self.forvo_scraping = True
-            self.audio_path = play_forvo(word, language)
-            self.forvo_scraping = False
         try:
             item = lookupin(word, language, lemmatize, dictname, gtrans_lang)
             self.rec.recordLookup(word, item['definition'], TL, lemmatize, dictionaries.get(dictname, dictname), True)
@@ -362,6 +379,7 @@ class DictionaryWindow(QMainWindow):
             self.definition2.clear()
             return item
         return {"word": item['word'], 'definition': item['definition'], 'definition2': item2['definition']}
+        
 
     def createNote(self):
         sentence = self.sentence.toPlainText().replace("\n", "<br>")
